@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: http_ntlm.c,v 1.42 2005/03/14 15:51:10 bagder Exp $
+ * $Id: http_ntlm.c,v 1.47 2005/10/13 07:57:51 bagder Exp $
  ***************************************************************************/
 #include "setup.h"
 
@@ -30,8 +30,7 @@
 */
 
 #ifndef CURL_DISABLE_HTTP
-#if defined(USE_SSLEAY) || defined(USE_WINDOWS_SSPI)
-/* We need OpenSSL for the crypto lib to provide us with MD4 and DES */
+#ifdef USE_NTLM
 
 /* -- WIN32 approved -- */
 #include <stdio.h>
@@ -388,6 +387,7 @@ CURLcode Curl_output_ntlm(struct connectdata *conn,
     ULONG attrs;
     const char *user;
     int domlen;
+    TimeStamp tsDummy; /* For Windows 9x compatibility of SPPI calls */
 
     ntlm_sspi_cleanup(ntlm);
 
@@ -431,8 +431,8 @@ CURLcode Curl_output_ntlm(struct connectdata *conn,
 
     if (AcquireCredentialsHandle(
           NULL, (char *)"NTLM", SECPKG_CRED_OUTBOUND, NULL, ntlm->p_identity,
-          NULL, NULL, &ntlm->handle, NULL
-        ) != SEC_E_OK) {
+          NULL, NULL, &ntlm->handle, &tsDummy
+          ) != SEC_E_OK) {
       return CURLE_OUT_OF_MEMORY;
     }
 
@@ -448,12 +448,23 @@ CURLcode Curl_output_ntlm(struct connectdata *conn,
                                        ISC_REQ_REPLAY_DETECT |
                                        ISC_REQ_CONNECTION,
                                        0, SECURITY_NETWORK_DREP, NULL, 0,
-                                       &ntlm->c_handle, &desc, &attrs, NULL
-                                      );
+                                       &ntlm->c_handle, &desc, &attrs, &tsDummy
+      );
 
     if (status == SEC_I_COMPLETE_AND_CONTINUE ||
         status == SEC_I_CONTINUE_NEEDED) {
-      CompleteAuthToken(&ntlm->c_handle, &desc);
+      /* CompleteAuthToken() is not present in Win9x, so load it dynamically */
+      SECURITY_STATUS (__stdcall * pCompleteAuthToken)
+        (PCtxtHandle,PSecBufferDesc);
+      HMODULE hSecur32 = GetModuleHandle("secur32.dll");
+      if (hSecur32 != NULL) {
+        pCompleteAuthToken =
+          (SECURITY_STATUS (__stdcall *)(PCtxtHandle,PSecBufferDesc))
+            GetProcAddress(hSecur32, "CompleteAuthToken");
+        if( pCompleteAuthToken != NULL ) {
+          pCompleteAuthToken(&ntlm->c_handle, &desc);
+        }
+      }
     }
     else if (status != SEC_E_OK) {
       FreeCredentialsHandle(&ntlm->handle);
@@ -554,6 +565,7 @@ CURLcode Curl_output_ntlm(struct connectdata *conn,
     SecBufferDesc type_2_desc, type_3_desc;
     SECURITY_STATUS status;
     ULONG attrs;
+    TimeStamp tsDummy; /* For Windows 9x compatibility of SPPI calls */
 
     type_2_desc.ulVersion  = type_3_desc.ulVersion  = SECBUFFER_VERSION;
     type_2_desc.cBuffers   = type_3_desc.cBuffers   = 1;
@@ -574,7 +586,7 @@ CURLcode Curl_output_ntlm(struct connectdata *conn,
                                        ISC_REQ_CONNECTION,
                                        0, SECURITY_NETWORK_DREP, &type_2_desc,
                                        0, &ntlm->c_handle, &type_3_desc,
-                                       &attrs, NULL);
+                                       &attrs, &tsDummy);
 
     if (status != SEC_E_OK)
       return CURLE_RECV_ERROR;
@@ -701,6 +713,13 @@ CURLcode Curl_output_ntlm(struct connectdata *conn,
     size=64;
     ntlmbuf[62]=ntlmbuf[63]=0;
 
+    /* Make sure that the user and domain strings fit in the target buffer
+       before we copy them there. */
+    if(size + userlen + domlen >= sizeof(ntlmbuf)) {
+      failf(conn->data, "user + domain name too big");
+      return CURLE_OUT_OF_MEMORY;
+    }
+
     memcpy(&ntlmbuf[size], domain, domlen);
     size += domlen;
 
@@ -769,5 +788,5 @@ Curl_ntlm_cleanup(struct connectdata *conn)
 #endif
 }
 
-#endif /* USE_SSLEAY */
+#endif /* USE_NTLM */
 #endif /* !CURL_DISABLE_HTTP */
